@@ -28,6 +28,8 @@
 #define IEEE80211_STYPE_AUTH		0xb
 #define IEEE80211_STYPE_DEAUTH		0xc
 #define IEEE80211_STYPE_ACTION		0xd
+#define IEEE80211_STYPE_NACK		0xe
+#define IEEE80211_STYPE_MGTRESERVED	0xf
 
 /* control */
 #define IEEE80211_STYPE_CTL_EXT		0x6
@@ -104,6 +106,48 @@
 #define WPA_KEY_INFO_REQUEST WBIT(11)
 #define WPA_KEY_INFO_ENCR_KEY_DATA WBIT(12) /* IEEE 802.11i/RSN only */
 /*===========================================================================*/
+enum ieee80211_radiotap_presence
+{
+ IEEE80211_RADIOTAP_TSFT		 = 0x00000001,
+ IEEE80211_RADIOTAP_FLAGS		 = 0x00000002,
+ IEEE80211_RADIOTAP_RATE		 = 0x00000004,
+ IEEE80211_RADIOTAP_CHANNEL		 = 0x00000008,
+ IEEE80211_RADIOTAP_FHSS		 = 0x00000010,
+ IEEE80211_RADIOTAP_DBM_ANTSIGNAL	 = 0x00000020,
+ IEEE80211_RADIOTAP_DBM_ANTNOISE	 = 0x00000040,
+ IEEE80211_RADIOTAP_LOCK_QUALITY	 = 0x00000080,
+ IEEE80211_RADIOTAP_TX_ATTENUATION	 = 0x00000100,
+ IEEE80211_RADIOTAP_DB_TX_ATTENUATION	 = 0x00000200,
+ IEEE80211_RADIOTAP_DBM_TX_POWER	 = 0x00000400,
+ IEEE80211_RADIOTAP_ANTENNA		 = 0x00000800,
+ IEEE80211_RADIOTAP_DB_ANTSIGNAL	 = 0x00001000,
+ IEEE80211_RADIOTAP_DB_ANTNOISE		 = 0x00002000,
+ IEEE80211_RADIOTAP_RX_FLAGS		 = 0x00004000,
+ IEEE80211_RADIOTAP_TX_FLAGS		 = 0x00008000,
+ IEEE80211_RADIOTAP_RTS_RETRIES		 = 0x00010000,
+ IEEE80211_RADIOTAP_DATA_RETRIES	 = 0x00020000,
+ /* 18 is XChannel, not defined */
+ IEEE80211_RADIOTAP_MCS			 = 0x00080000,
+ IEEE80211_RADIOTAP_AMPDU_STATUS	 = 0x00100000,
+ IEEE80211_RADIOTAP_VHT			 = 0x00200000,
+ IEEE80211_RADIOTAP_TIMESTAMP		 = 0x00400000,
+ /* valid in every it_present bitmap, even vendor namespaces */
+ IEEE80211_RADIOTAP_RADIOTAP_NAMESPACE	 = 0x20000000,
+ IEEE80211_RADIOTAP_VENDOR_NAMESPACE	 = 0x40000000,
+ IEEE80211_RADIOTAP_EXT			 = 0x80000000,
+};
+
+enum ieee80211_radiotap_flags
+{
+ IEEE80211_RADIOTAP_F_CFP = 0x01,
+ IEEE80211_RADIOTAP_F_SHORTPRE = 0x02,
+ IEEE80211_RADIOTAP_F_WEP = 0x04,
+ IEEE80211_RADIOTAP_F_FRAG = 0x08,
+ IEEE80211_RADIOTAP_F_FCS = 0x10,
+ IEEE80211_RADIOTAP_F_DATAPAD = 0x20,
+ IEEE80211_RADIOTAP_F_BADFCS = 0x40,
+};
+
 struct radiotap_header
 {
  uint8_t	it_version;
@@ -125,6 +169,11 @@ typedef struct ethernet2_header eth2_t;
 /*===========================================================================*/
 struct loopback_header
 {
+#define LOBA_IPV4	2
+#define LOBA_IPX	23
+#define LOBA_IPV624	24
+#define LOBA_IPV628	28
+#define LOBA_IPV630	30
  uint32_t		family;
 } __attribute__((packed));
 typedef struct loopback_header loba_t;
@@ -317,9 +366,11 @@ struct ie_tag
 #define	TAG_SSID	0x00
 #define	TAG_RATE	0x01
 #define	TAG_CHAN	0x03
+#define	TAG_COUNTRY	0x07
 #define	TAG_RSN		0x30
 #define	TAG_PAG		0xde
 #define	TAG_VENDOR	0xdd
+#define	TAG_EXT		0xff
  uint8_t		len;
  uint8_t		data[1];
 } __attribute__((__packed__));
@@ -354,6 +405,15 @@ static const uint8_t ouiwifialliance[3] =
 0x50, 0x6f, 0x9a
 };
 /*===========================================================================*/
+struct owe_tag
+{
+ uint8_t	number;
+ uint16_t	group;
+ uint8_t	data[1];
+} __attribute__((__packed__));
+typedef struct owe_tag owe_t;
+#define	OWE_SIZE offsetof(owe_t, data)
+/*===========================================================================*/
 struct rsnie_tag
 {
  uint16_t		version;
@@ -578,17 +638,17 @@ typedef struct rsnpmkidlist_s rsnpmkidlist_t;
 /*===========================================================================*/
 struct exteap_frame
 {
- uint8_t			code;
+ uint8_t	code;
 #define EAP_CODE_REQ		1
 #define EAP_CODE_RESP		2
 #define EAP_CODE_SUCCESS	3
 #define EAP_CODE_FAILURE	4
 #define EAP_CODE_INITIATE	5
 #define EAP_CODE_FINISH		6
- uint8_t			id;
+ uint8_t	id;
 #define EAP_TYPE_ID		1
- uint16_t			len;
- uint8_t			type;
+ uint16_t	len;
+ uint8_t	type;
 #define EAP_TYPE_EAP		0
 #define EAP_TYPE_ID		1
 #define EAP_TYPE_NOTIFY		2
@@ -644,16 +704,16 @@ struct exteap_frame
 #define EAP_TYPE_TEAP		55
 #define	EAP_TYPE_EXPAND		254
 #define EAP_TYPE_EXPERIMENTAL	255
- uint8_t			data[1];
+ uint8_t	data[1];
 } __attribute__((__packed__));
 typedef struct exteap_frame exteap_t;
 #define	EXTEAP_SIZE offsetof(exteap_t, data)
 /*===========================================================================*/
 struct mpdu_frame
 {
- uint8_t pn[3];
- uint8_t keyid;
- uint8_t exitiv[4];
+ uint8_t	pn[3];
+ uint8_t	keyid;
+ uint8_t	exitiv[4];
 };
 typedef struct mpdu_frame mpdu_t;
 #define	MPDU_SIZE (sizeof(mpdu_t))
@@ -684,6 +744,23 @@ struct eapleap_frame
 typedef struct eapleap_frame eapleap_t;
 #define	EAPLEAP_SIZE offsetof(eapleap_t, leapdata)
 /*===========================================================================*/
+struct eapmschapv2_frame
+{
+ uint8_t	code;
+ uint8_t	id;
+ uint16_t	eaplen;
+ uint8_t	type;
+ uint8_t	opcode;
+ #define EAP_MSCHAPV2_OPCODE_REQ 1
+ #define EAP_MSCHAPV2_OPCODE_RESP 2
+ uint8_t	mschapv2id;
+ uint16_t	mschapv2len;
+ uint8_t	mschapv2valuesize;
+ uint8_t	mschapv2data[1];
+} __attribute__((__packed__));
+typedef struct eapmschapv2_frame eapmschapv2_t;
+#define	EAPMSCHAPV2_SIZE offsetof(eapmschapv2_t, mschapv2data)
+/*===========================================================================*/
 struct ipv4_frame
 {
  uint8_t	ver_hlen;
@@ -787,7 +864,6 @@ struct tzsp_tag
  uint8_t	tag;
 #define TZSP_TAG_END 1
 #define TZSP_TAG_ORGLEN 41
-
  uint8_t	len;
  uint8_t	data[1];
 } __attribute__ ((packed));
@@ -820,9 +896,9 @@ typedef struct ptp_frame ptp_t;
 struct chap_frame
 {
  uint8_t	code;
-#define	CHAP_CODE_REQ	1
-#define	CHAP_CODE_RESP	2
-
+#define	CHAP_CODE_REQ		1
+#define	CHAP_CODE_RESP		2
+#define	CHAP_CODE_SUCCESS	3
  uint8_t	id;
  uint16_t	len;
  uint8_t	data[1];
@@ -832,15 +908,15 @@ typedef struct chap_frame chap_t;
 /*===========================================================================*/
 struct tacacsp_frame
 {
- uint8_t   version;
+ uint8_t	version;
 #define TACACSP_VERSION 0xc0
- uint8_t   type;
+ uint8_t	type;
 #define TACACS_AUTHENTICATION 1
- uint8_t   sequencenr;
- uint8_t   flags;
- uint32_t  sessionid;
- uint32_t  len;
- uint8_t   data[1];
+ uint8_t	sequencenr;
+ uint8_t	flags;
+ uint32_t	sessionid;
+ uint32_t	len;
+ uint8_t	data[1];
 } __attribute__ ((packed));
 typedef struct tacacsp_frame tacacsp_t;
 #define	TACACSP_SIZE offsetof(tacacsp_t, data)
